/*
 * Copyright (c) 1993 Addison-Wesley Publishing Company.
 * All rights reserved.  This source code is provided
 * for its instructional value, and is not guaranteed
 * for any particular purpose.
 */
#include <sys/types.h>
#include <tiuser.h>
#include <errno.h>
#include <unistd.h>
#include <stdio.h>

#define MAXSLP	32

int
connect(int fd, char *addr, uint_t alen)
{
	struct t_call *callp;
	int i;
	extern int t_errno;

	/*
	 * Allocate a t_call structure for the connect
	 * request and point the destination address at
	 * the address passed in by the caller.
	 */
	callp = (struct t_call *)t_alloc(fd, T_CALL, 0);
	if (callp == NULL) {
		t_error("Cannot allocate call structure");
		return(-1);
	}
	callp->addr.buf = addr;
	callp->addr.len = alen;

	/*
	 * Try several times to connect.
	 */
	for (i = 1; i <= MAXSLP; i <<= 1) {
		if (t_connect(fd, callp, NULL) == 0) {
			/*
			 * Connection accepted.
			 */
			callp->addr.buf = NULL;
			t_free(callp, T_CALL);
			return(0);
		}
		if (t_errno == TLOOK) {
			/*
			 * Connection rejected.
			 */
			t_rcvdis(fd, NULL);
		} else {
			/*
			 * Fatal error.
			 */
			t_error("Cannot connect");
			callp->addr.buf = NULL;
			t_free(callp, T_CALL);
			return(-1);
		}

		/*
		 * Delay before trying again.
		 */
		sleep(i);
	}

	/*
	 * We reached the maximum retry count.  Free the
	 * t_call structure, print an error message, set
	 * the global error codes, and return -1.
	 */
	callp->addr.buf = NULL;
	t_free(callp, T_CALL);
	fprintf(stderr, "Connect timed out\n");
	t_errno = TSYSERR;
	errno = ETIME;
	return(-1);
}
